@@ -42,7 +42,7 @@
  */
 
 typedef unsigned renderer_t;
-enum renderer_e {GS = 0, PDFTOPS = 1, ACROREAD = 2, PDFTOCAIRO = 3};
+enum renderer_e {GS = 0, PDFTOPS = 1, ACROREAD = 2, PDFTOCAIRO = 3, HYBRID = 4};
 
 /*
  * Local functions...
@@ -231,7 +231,7 @@ int					/* O - Exit status */
 main(int  argc,				/* I - Number of command-line args */
      char *argv[])			/* I - Command-line arguments */
 {
-  renderer_t    renderer = CUPS_PDFTOPS_RENDERER; /* Renderer: gs or pdftops or acroread */
+  renderer_t    renderer = CUPS_PDFTOPS_RENDERER; /* Renderer: gs or pdftops or acroread or pdftocairo or hybrid */
   int		fd = 0;			/* Copy file descriptor */
   char		*filename,		/* PDF file to convert */
 		tempfile[1024];		/* Temporary file */
@@ -371,7 +371,10 @@ main(int  argc,				/* I - Number of command-line args */
   cupsMarkOptions(ppd, num_options, options);
 
  /*
-  * Select the PDF renderer: Ghostscript (gs) or Poppler (pdftops)
+  * Select the PDF renderer: Ghostscript (gs), Poppler (pdftops),
+  * Adobe Reader (arcoread), Poppler with Cairo (pdftocairo), or
+  * Hybrid (hybrid, Poppler for Brother, Minolta, and Konica Minolta and
+  * Ghostscript otherwise)
   */
 
   if ((val = cupsGetOption("pdftops-renderer", num_options, options)) != NULL)
@@ -384,11 +387,26 @@ main(int  argc,				/* I - Number of command-line args */
       renderer = ACROREAD;
     else if (strcasecmp(val, "pdftocairo") == 0)
       renderer = PDFTOCAIRO;
+    else if (strcasecmp(val, "hybrid") == 0)
+      renderer = HYBRID;
     else
       fprintf(stderr,
 	      "WARNING: Invalid value for \"pdftops-renderer\": \"%s\"\n", val);
   }
 
+  if (renderer == HYBRID)
+  {
+    if (ppd && ppd->manufacturer &&
+	(!strncasecmp(ppd->manufacturer, "Brother", 7) ||
+	 strcasestr(ppd->manufacturer, "Minolta")))
+      {
+	fprintf(stderr, "DEBUG: Switching to Poppler's pdftops instead of Ghostscript for Brother, Minolta, and Konica Minolta to work around bugs in the printer's PS interpreters\n");
+	renderer = PDFTOPS;
+      }
+    else
+      renderer = GS;
+  }
+
  /*
   * Build the pstops command-line...
   */
